@@ -18,8 +18,11 @@ except ImportError:
     from collections import Sequence
 from itertools import chain
 
-from .exceptions import IncorrectCellLabel, NoValidUrlKeyFound
+from google.auth.credentials import Credentials as Credentials
+from google.oauth2.credentials import Credentials as UserCredentials
+from google.oauth2.service_account import Credentials as ServiceAccountCredentials
 
+from .exceptions import IncorrectCellLabel, NoValidUrlKeyFound
 
 if sys.version_info.major == 2:
     import urllib
@@ -40,6 +43,46 @@ URL_KEY_V1_RE = re.compile(r'key=([^&#]+)')
 URL_KEY_V2_RE = re.compile(r'/spreadsheets/d/([a-zA-Z0-9-_]+)')
 
 
+def convert_credentials(credentials):
+    module = credentials.__module__
+    cls = credentials.__class__.__name__
+    if 'oauth2client' in module and cls == 'ServiceAccountCredentials':
+        return _convert_service_account(credentials)
+    elif 'oauth2client' in module and cls == 'OAuth2Credentials':
+        return _convert_oauth(credentials)
+    elif isinstance(credentials, Credentials):
+        return credentials
+
+    raise TypeError(
+        'Credentials need to be from either oauth2client or from google-auth.'
+    )
+
+
+def _convert_oauth(credentials):
+    return UserCredentials(
+        credentials.access_token,
+        credentials.refresh_token,
+        credentials.id_token,
+        credentials.token_uri,
+        credentials.client_id,
+        credentials.client_secret,
+        credentials.scopes,
+    )
+
+
+def _convert_service_account(credentials):
+    data = credentials.serialization_data
+    data['token_uri'] = credentials.token_uri
+    scopes = credentials._scopes.split() or [
+        'https://www.googleapis.com/auth/drive',
+        'https://spreadsheets.google.com/feeds',
+    ]
+
+    return ServiceAccountCredentials.from_service_account_info(
+        data, scopes=scopes
+    )
+
+
 def finditem(func, seq):
     """Finds and returns first item in iterable for which func(item) is True.
 
@@ -80,8 +123,10 @@ def numericise(value, empty2zero=False, default_blank="", allow_underscores_in_n
     >>>
     """
     if value is not None:
-        if "_" in value and not allow_underscores_in_numeric_literals:
-            return value
+        if "_" in value:
+            if not allow_underscores_in_numeric_literals:
+                return value
+            value = value.replace("_", "")
         try:
             value = int(value)
         except ValueError:
@@ -298,12 +343,12 @@ def cast_to_a1_notation(method):
             if len(args):
                 int(args[0])
 
-            # Convert to A1 notation
-            range_start = rowcol_to_a1(*args[:2])
-            range_end = rowcol_to_a1(*args[-2:])
-            range_name = ':'.join((range_start, range_end))
+                # Convert to A1 notation
+                range_start = rowcol_to_a1(*args[:2])
+                range_end = rowcol_to_a1(*args[-2:])
+                range_name = ':'.join((range_start, range_end))
 
-            args = (range_name,) + args[4:]
+                args = (range_name,) + args[4:]
         except ValueError:
             pass
 
@@ -380,17 +425,37 @@ def quote(value, safe='', encoding='utf-8'):
     return urllib.quote(value.encode(encoding), safe)
 
 
-def absolute_range_name(sheet_name, range_name):
+def absolute_range_name(sheet_name, range_name=None):
     """Return an absolutized path of a range.
 
-    >>> absolute_range_name('Sheet1', 'A1:B1')
-    'Sheet1!A1:B1'
+    >>> absolute_range_name("Sheet1", "A1:B1")
+    "'Sheet1'!A1:B1"
 
-    >>> absolute_range_name('Sheet1', 'A1')
-    'Sheet1!A1'
+    >>> absolute_range_name("Sheet1", "A1")
+    "'Sheet1'!A1"
+
+    >>> absolute_range_name("Sheet1")
+    "'Sheet1'"
+
+    >>> absolute_range_name("Sheet'1")
+    "'Sheet''1'"
+
+    >>> absolute_range_name("Sheet''1")
+    "'Sheet''''1'"
+
+    >>> absolute_range_name("''sheet12''", "A1:B2")
+    "'''''sheet12'''''!A1:B2"
 
     """
-    return '%s!%s' % (sheet_name, range_name)
+
+    sheet_name = "'{}'".format(
+        sheet_name.replace("'", "''")
+    )
+
+    if range_name:
+        return '{}!{}'.format(sheet_name, range_name)
+    else:
+        return sheet_name
 
 
 def is_scalar(x):
